var FC = (function (FC, window, $) {
    (function (Map, window, $) {
        "use strict";

        var $geoPanel,
            $d3Div,
            $mapDiv,
            mapKey,
            plot,
            map,
            locality = null;

        Object.defineProperty(Map, "plot", {
            get: function () {
                return plot;
            }
        });

        Object.defineProperty(Map, "map", {
            get: function () {
                return map;
            }
        });

        Object.defineProperty(Map, "locality", {
            get: function () {
                return locality;
            },
            set: function (value) {
                locality = value;
            }
        });

        Map.init = function () {
            // Set visible region so map fills entire screen. NOT WORKING. Probably, bug in D3
            var pixWidth = $(document).width();
            var pixHeight = $("#map").height()-2;

            $geoPanel = $(".geo-panel");
            $d3Div = $("#geo-plot");
            $mapDiv = $("#geo-map");
            mapKey = $mapDiv.attr("data-d3-mapKey");
            plot = D3.asPlot($d3Div);
            map = new Microsoft.Maps.Map($mapDiv[0], {
                credentials: mapKey,
                mapTypeId: Microsoft.Maps.MapTypeId.road,
                enableClickableLogo: false,
                enableSearchLogo: false,
                showCopyright: false,
                showDashboard: false,
                showLogo: false,
                disablePanning: false,
                disableZooming: false,
                width: $mapDiv.width(),
                height: $mapDiv.height(),

                zoom:1,
                center: new Microsoft.Maps.Location(51.5072, -0.1275)
            });

            $d3Div.appendTo($(map.getModeLayer()));

            Microsoft.Maps.Events.addHandler(map, 'viewchangeend', FC.Map.updateView);

            Microsoft.Maps.Events.addHandler(map, 'viewchangeend', function () {
                updateChart();
                FC.Map.locality = updateName();
            });

            Microsoft.Maps.Events.addHandler(map, 'viewchange', function () {
                updateChart();
                FC.Map.locality = updateCoords();
            });

            map.setOptions({ width: pixWidth, height: pixHeight });

            updateChart();
        };

        Map.updateChartOnMapLoad = function () {
            var watch = function () {
                if ($geoPanel.is(":visible")) {
                    updateChart();
                } else {
                    setTimeout(watch, 50);
                }
            };
            setTimeout(watch, 50);
        };

        /**
         * Enables navigation of Bing Maps.
         */
        Map.enableNavigation = function () {
            map.setOptions({
                disablePanning: false,
                disableZooming: false
            });
        };

        /**
         * Disables navigation of Bing Maps.
         */
        Map.disableNavigation = function () {
            map.setOptions({
                disablePanning: true,
                disableZooming: true
            });
        };

        /**
         * Updates size and position of d3 master plot such that it covers Bing maps visible area.
         */
        var updateChart = function () {
            var bounds = map.getBounds();
            var mapCenter = map.getCenter();
            var mapWidth = map.getWidth();
            var mapHeight = map.getHeight();

            var deltaLon = 30;
            var firstPoint = map.tryLocationToPixel({ latitude: 0, longitude: mapCenter.longitude }, Microsoft.Maps.PixelReference.control);
            var secondPoint = map.tryLocationToPixel({ latitude: 0, longitude: mapCenter.longitude + deltaLon }, Microsoft.Maps.PixelReference.control);
            var pixelDelta = secondPoint.x - firstPoint.x;

            if (pixelDelta < 0)
                pixelDelta = firstPoint.x - map.tryLocationToPixel({ latitude: 0, longitude: mapCenter.longitude - deltaLon }, Microsoft.Maps.PixelReference.control).x;

            var pixelsInDegree = pixelDelta / deltaLon;
            var degs = mapWidth / pixelsInDegree;
            var left = mapCenter.longitude - degs / 2;
            var right = mapCenter.longitude + degs / 2;

            var topPixel = map.tryLocationToPixel({ latitude: bounds.getNorth(), longitude: mapCenter.longitude }, Microsoft.Maps.PixelReference.control);
            var bottomPixel = map.tryLocationToPixel({ latitude: bounds.getSouth(), longitude: mapCenter.longitude }, Microsoft.Maps.PixelReference.control);

            $d3Div.css('left', -mapWidth / 2);
            $d3Div.css('top', -mapHeight / 2 + topPixel.y);
            $d3Div.width(mapWidth);
            $d3Div.height(bottomPixel.y - topPixel.y);

            var top = bounds.getNorth();
            var bottom = bounds.getSouth();

            plot.navigation.setVisibleRect({
                x: left,
                y: D3.mercatorTransform.dataToPlot(bottom),
                width: degs,
                height: D3.mercatorTransform.dataToPlot(top) - D3.mercatorTransform.dataToPlot(bottom)
                }, false);

            plot.updateLayout();
        };

        // @MOBILE
        FC.Map.updateView = function () {
            var bounds = map.getBounds(),
                center = map.getCenter(),
                lat = center.latitude,
                latmin = bounds.getSouth(),
                latmax = bounds.getNorth(),
                lon = center.longitude,
                lonmin = bounds.getWest(),
                lonmax = bounds.getEast(),
                timestamp = Date.now(),
                temporal, grid, tilesNumber;

            if (lonmin > lonmax) lonmax = center.longitude > 0 ? 180 : -180;

            //console.time("phoneData")
            tilesNumber = FC.phoneData(FC.variable, lat, latmin, latmax, lon, lonmin, lonmax, function (data) {
                //console.time("phoneDataInside")
                data.timestamp = timestamp;
                drawMinMaxIndicators(data);

                if (data.id >= 0) {
                    FC.heatmaps.drawHeatmap(data);
                }
                //console.timeEnd("phoneDataInside")
            });
            //console.timeEnd("phoneData")

            FC.heatmaps.hideUnnecessaryHeatmaps(tilesNumber);
        }

        FC.Map.HeatmapCollection = function (count) {
            var _heatmaps = [],
                _queue = [],
                _fps = 2,
                _interval = 1000 / _fps,
                _now = Date.now(),
                _then = Date.now(),
                _delta, i, h;

            Object.defineProperties(this, {
                children: { get: function () { return _heatmaps } },
                visible: { get: function () { return _heatmaps.filter(function (h) { return h.isVisible; }); } },
                invisible: { get: function () { return _heatmaps.filter(function (h) { return !h.isVisible; }); } }
            });

            function init() {
                for (i = 0; i < count; ++i) {
                    h = plot.heatmap("heatmap" + i);
                    h.yDataTransform = D3.mercatorTransform;
                    h.isToolTipEnabled = false;
                    h.isVisible = false;
                    _heatmaps.push(h);
                }

                // NOTE: Use it in pair with FC.heatmaps.enqueue() method.
                //       Requires level and tile in incoming data.
                // startRenderingLoop();
            }

            function startRenderingLoop() {
                function loop() {
                    requestAnimationFrame(loop);
                    _now = Date.now();
                    _delta = _now - _then;

                    if (_delta > _interval) {
                        _then = _now - (_delta % _interval);
                        render();
                    }
                }
                loop();
            }

            function render() {
                var level, groups, data,
                    dataToDraw, palette, h;

                if (_queue.length > 0) {
                    level = getLevel();
                    groups = {};

                    do {
                        data = _queue.pop();
                        groups[data.level] = groups[data.level] || [];
                        groups[data.level].push(data);
                    } while (_queue.length > 0);

                    while (!groups[level] && level > 0) {
                        level--;
                    }

                    dataToDraw = groups[level] || [];
                    palette = D3.ColorPalette.parse(FC.palette);
                    palette = palette.absolute(FC.range.min, FC.range.max);

                    dataToDraw.forEach(function (d) {
                        var hd = {
                            x: d.lons,
                            y: d.lats,
                            f: getGridSlice(d.grid, FC.month)
                        };

                        hideHeatmaps(d, level);
                        h = getAvailableHeatmap();
                        h.palette = palette;
                        h.draw(hd);
                        h.isVisible = true;

                        h.level = d.level;
                        h.tile = d.tile.toString();
                    });

                    updateChart();
                }
            }

            function getLevel() {
                var bounds = map.getBounds(),
                    latmin = bounds.getSouth(),
                    latmax = bounds.getNorth(),
                    lonmin = bounds.getWest(),
                    lonmax = bounds.getEast();

                if (latmin > latmax) latmax = [latmin, latmin = latmax][0];
                if (lonmin > lonmax) lonmax = [lonmin, lonmin = lonmax][0];

                var g_n = 64, // base tile resolution in intervals
                    g_h = 180.0 / 32 / g_n, // the lowest step at level 5
                    g_view = 48 * g_h,
                    n = Math.max(latmax - latmin, Math.abs(lonmax - lonmin)) / g_view;

                return n < 1 ? 5 : n < 2 ? 4 : n < 4 ? 3 : n < 8 ? 2 : n < 16 ? 1 : 0;
            }

            function getAvailableHeatmap() {
                return _heatmaps.filter(function (h) {
                    return !h.isVisible;
                })[0];
            }

            function hideHeatmaps(data, level) {
                _heatmaps.forEach(function (h) {
                    if (h.level !== level || h.tile === data.tile.toString()) {
                        h.isVisible = false;
                    }
                });
            }

            this.enqueue = function (data) {
                _queue.push(data);
            };

            this.hideUnnecessaryHeatmaps = function (tilesNumber) {
                var numToHide = _heatmaps.length - tilesNumber;
                if (numToHide > 0) _heatmaps.slice(-numToHide).forEach(function (h) {
                    h.isVisible = false;
                });
            };

            this.drawHeatmap = function (data) {
                var id = data.id,
                    heatmapData,
                    palette;

                if (_heatmaps[id].timestamp > data.timestamp) return;

                heatmapData = {
                    x: data.lons,
                    y: data.lats,
                    f: getGridSlice(data.grid, FC.month)
                };

                palette = D3.ColorPalette.parse(FC.palette);
                palette = palette.absolute(FC.range.min, FC.range.max);

                _heatmaps[id].timestamp = data.timestamp;
                _heatmaps[id].palette = palette;
                _heatmaps[id].draw(heatmapData);
                _heatmaps[id].isVisible = true;

                updateChart();
            };

            init();
        };

        var getGridSlice = function (grid, month) {
            var slice,
                lat, lon,
                latlen, lonlen;

            lonlen = grid.length;
            slice = new Array(lonlen);

            for (lon = 0; lon < lonlen; lon++) {
                latlen = grid[lon].length;
                slice[lon] = new Array(latlen);

                for (lat = 0; lat < latlen; lat++) {
                    slice[lon][lat] = grid[lon][lat][month];
                }
            }

            convertNullToNaN(slice);

            return slice;
        };

        var convertNullToNaN = function (slice) {
            var lat, lon, latlen, lonlen;

            lonlen = slice.length;

            for (lon = 0; lon < lonlen; lon++) {
                latlen = slice[lon].length;

                for (lat = 0; lat < latlen; lat++) {
                    slice[lon][lat] = (slice[lon][lat] === null) ? NaN : slice[lon][lat];
                }
            }
        };

        return Map;
    })(FC.Map || (FC.Map = {}), window, $);

    return FC;
})(FC || {}, window, jQuery);
